@@ -30,7 +30,7 @@ import time
 import traceback
 import uuid
 
-from shakenfist import config
+from shakenfist.config import config
 from shakenfist import db
 from shakenfist import exceptions
 from shakenfist import logutil
@@ -63,7 +63,7 @@ def error(status_code, message):
         'status': status_code
     }
 
-    if TESTING or config.parsed.get('INCLUDE_TRACEBACKS') == '1':
+    if TESTING or config.get('INCLUDE_TRACEBACKS'):
         _, _, tb = sys.exc_info()
         if tb:
             body['traceback'] = traceback.format_exc()
@@ -71,8 +71,7 @@ def error(status_code, message):
     resp = flask.Response(json.dumps(body),
                           mimetype='application/json')
     resp.status_code = status_code
-    LOG.error('Returning API error: %d, %s\n    %s'
-              % (status_code, message, '\n    '.join(body.get('traceback', '').split('\n'))))
+    LOG.info('Returning API error: %d, %s' % (status_code, message))
     return resp
 
 
@@ -141,6 +140,7 @@ def generic_wrapper(func):
             return error(401, str(e))
 
         except Exception as e:
+            LOG.exception('Server error')
             return error(500, 'server error: %s' % repr(e))
 
     return wrapper
@@ -195,13 +195,13 @@ def redirect_instance_request(func):
     # Redirect method to the hypervisor hosting the instance
     def wrapper(*args, **kwargs):
         i = kwargs.get('instance_from_db_virt')
-        if i and i.db_entry['node'] != config.parsed.get('NODE_NAME'):
+        if i and i.db_entry['node'] != config.NODE_NAME:
             url = 'http://%s:%d%s' % (i.db_entry['node'],
-                                      config.parsed.get('API_PORT'),
+                                      config.get('API_PORT'),
                                       flask.request.environ['PATH_INFO'])
             api_token = util.get_api_token(
                 'http://%s:%d' % (i.db_entry['node'],
-                                  config.parsed.get('API_PORT')),
+                                  config.get('API_PORT')),
                 namespace=get_jwt_identity())
             r = requests.request(
                 flask.request.environ['REQUEST_METHOD'], url,
@@ -256,16 +256,16 @@ def arg_is_network_uuid(func):
 def redirect_to_network_node(func):
     # Redirect method to the network node
     def wrapper(*args, **kwargs):
-        if config.parsed.get('NODE_IP') != config.parsed.get('NETWORK_NODE_IP'):
+        if not util.is_network_node():
             admin_token = util.get_api_token(
-                'http://%s:%d' % (config.parsed.get('NETWORK_NODE_IP'),
-                                  config.parsed.get('API_PORT')),
+                'http://%s:%d' % (config.NETWORK_NODE_IP,
+                                  config.get('API_PORT')),
                 namespace='system')
             r = requests.request(
                 flask.request.environ['REQUEST_METHOD'],
                 'http://%s:%d%s'
-                % (config.parsed.get('NETWORK_NODE_IP'),
-                   config.parsed.get('API_PORT'),
+                % (config.NETWORK_NODE_IP,
+                   config.get('API_PORT'),
                    flask.request.environ['PATH_INFO']),
                 data=flask.request.data,
                 headers={'Authorization': admin_token,
@@ -318,7 +318,7 @@ def _metadata_putpost(meta_type, owner, key, value):
 
 app = flask.Flask(__name__)
 api = flask_restful.Api(app, catch_all_404s=False)
-app.config['JWT_SECRET_KEY'] = config.parsed.get('AUTH_SECRET_SEED')
+app.config['JWT_SECRET_KEY'] = config.AUTH_SECRET_SEED.get_secret_value()
 jwt = JWTManager(app)
 
 # Use our handler to get SF log format (instead of gunicorn's handlers)
@@ -444,11 +444,12 @@ class AuthNamespace(Resource):
             return error(400, 'you cannot delete a namespace with instances')
 
         networks = []
-        deleted_networks = []
         for n in db.get_networks(all=True, namespace=namespace):
-            if n['state'] in ['deleted', 'error']:
-                deleted_networks.append(n['uuid'])
-            else:
+            # Networks in 'deleting' state are regarded as "live" networks.
+            # They in a transient state. If they hang in that state we want to
+            # know. They will block deletion of a namespace thus giving notice
+            # of the problem.
+            if n['state'] not in ['deleted', 'error']:
                 networks.append(n['uuid'])
         if len(networks) > 0:
             return error(400, 'you cannot delete a namespace with networks')
@@ -582,14 +583,14 @@ class Instance(Resource):
 
         # If this instance is not on a node, just do the DB cleanup locally
         if not instance_from_db['node']:
-            node = config.parsed.get('NODE_NAME')
+            node = config.NODE_NAME
         else:
             node = instance_from_db['node']
 
         db.enqueue_instance_delete_remote(node, instance_from_db['uuid'])
 
         start_time = time.time()
-        while time.time() - start_time < config.parsed.get('API_ASYNC_WAIT'):
+        while time.time() - start_time < config.get('API_ASYNC_WAIT'):
             i = db.get_instance(instance_uuid)
             if i['state'] in ['deleted', 'error']:
                 return
@@ -634,6 +635,15 @@ class Instances(Resource):
                 if 'network_uuid' not in n:
                     return error(400, 'network specification is missing network_uuid')
 
+                if n.get('address'):
+                    # The requested address must be within the ip range specified
+                    # for that virtual network
+                    ipm = db.get_ipmanager(n['network_uuid'])
+                    if not ipm.is_in_range(n['address']):
+                        return error(400,
+                                     'network specification requests an address outside the '
+                                     'range of the network')
+
         if not video:
             video = {'model': 'cirrus', 'memory': 16384}
 
@@ -761,7 +771,7 @@ class Instances(Resource):
         # Watch for a while and return results if things are fast, give up
         # after a while and just return the current state
         start_time = time.time()
-        while time.time() - start_time < config.parsed.get('API_ASYNC_WAIT'):
+        while time.time() - start_time < config.get('API_ASYNC_WAIT'):
             i = db.get_instance(instance_uuid)
             if i['state'] in ['created', 'deleted', 'error']:
                 return i
@@ -795,7 +805,7 @@ class Instances(Resource):
 
             # If this instance is not on a node, just do the DB cleanup locally
             if not instance['node']:
-                node = config.parsed.get('NODE_NAME')
+                node = config.NODE_NAME
             else:
                 node = instance['node']
 
@@ -808,7 +818,8 @@ class Instances(Resource):
 
         waiting_for = copy.copy(instances_del)
         start_time = time.time()
-        while (waiting_for and (time.time() - start_time < config.parsed.get('API_ASYNC_WAIT'))):
+        while (waiting_for and
+               (time.time() - start_time < config.get('API_ASYNC_WAIT'))):
             for instance_uuid in copy.copy(waiting_for):
                 i = db.get_instance(instance_uuid)
                 if i['state'] in ['deleted', 'error']:
@@ -1107,7 +1118,7 @@ class Images(Resource):
     @jwt_required
     def post(self, url=None):
         db.add_event('image', url, 'api', 'cache', None, None)
-        db.enqueue(config.parsed.get('NODE_NAME'), {
+        db.enqueue(config.NODE_NAME, {
             'tasks': [FetchImageTask(url)],
         })
 
@@ -1120,20 +1131,30 @@ class ImageEvents(Resource):
 
 
 def _delete_network(network_from_db):
-    network_uuid = network_from_db['uuid']
-    db.add_event('network', network_uuid, 'api', 'delete', None, None)
+    # Load network from DB to ensure obtaining correct lock.
+    n = net.from_db(network_from_db['uuid'])
+
+    with db.get_object_lock(n, ttl=120, op='Network deleting'):
+        if not n or n.is_dead():
+            LOG.withFields({'network_uuid': n.db_entry['uuid'],
+                            'state': n.db_entry['state']}).warning(
+                                'delete_network: network does not exist')
+            return error(404, 'network is deleted')
+        db.update_network_state(n.db_entry['uuid'], 'deleting')
+
+    db.add_event('network', n.db_entry['uuid'], 'api', 'delete', None, None)
 
-    n = net.from_db(network_uuid)
     n.remove_dhcp()
     n.delete()
 
     if n.db_entry.get('floating_gateway'):
-        with db.get_lock('ipmanager', None, 'floating', ttl=120, op='Network delete'):
+        with db.get_lock(
+                'ipmanager', None, 'floating', ttl=120, op='Network delete'):
             ipm = db.get_ipmanager('floating')
             ipm.release(n.db_entry['floating_gateway'])
             db.persist_ipmanager('floating', ipm.save())
 
-    db.update_network_state(network_uuid, 'deleted')
+    db.update_network_state(n.db_entry['uuid'], 'deleted')
 
 
 class Network(Resource):
@@ -1161,7 +1182,7 @@ class Network(Resource):
         if network_from_db['state'] in 'deleted':
             return error(404, 'network not found')
 
-        _delete_network(network_from_db)
+        return _delete_network(network_from_db)
 
 
 class Networks(Resource):
@@ -1191,8 +1212,9 @@ class Networks(Resource):
 
         # If accessing a foreign name namespace, we need to be an admin
         if get_jwt_identity() not in [namespace, 'system']:
-            return error(401,
-                         'only admins can create resources in a different namespace')
+            return error(
+                401,
+                'only admins can create resources in a different namespace')
 
         network = db.allocate_network(netblock, provide_dhcp,
                                       provide_nat, name, namespace)
