@@ -8,7 +8,7 @@ import re
 import setproctitle
 import traceback
 
-from shakenfist import config
+from shakenfist.config import config
 
 
 # These classes are extensions of the work in https://github.com/vmig/pylogrus
@@ -106,7 +106,7 @@ class SFCustomAdapter(logging.LoggerAdapter, PyLogrusBase):
         msg = '%s[%s] %s' % (setproctitle.getproctitle(), os.getpid(), msg)
         kwargs["extra"] = self.extra
 
-        if config.parsed.get('LOG_METHOD_TRACE'):
+        if config.get('LOG_METHOD_TRACE'):
             # Determine the name of the calling method
             filename = traceback.extract_stack()[-4].filename
             f_match = self.FILENAME_RE.match(filename)
