@@ -5,9 +5,10 @@ import psutil
 import re
 
 
-from shakenfist import config
+from shakenfist.config import config
 from shakenfist import db
 from shakenfist import dhcp
+from shakenfist.exceptions import DeadNetwork
 from shakenfist import logutil
 from shakenfist.tasks import (DeployNetworkTask,
                               UpdateDHCPNetworkTask,
@@ -19,13 +20,20 @@ LOG, _ = logutil.setup(__name__)
 
 
 def from_db(uuid):
-    dbnet = db.get_network(uuid)
-    if not dbnet:
-        return None
-    if dbnet['state'] == 'deleted':
-        LOG.withNetwork(uuid).info('Network is deleted, returning None.')
-        return None
-    return Network(dbnet)
+    # TODO(andy): The whole system of unlocked in-memory objects needs to be
+    # revisited. This lock avoids the network being deleted between the DB load
+    # in the get_network() call and Network.__init__() loading the IPManager
+    # from the DB. Under extreme load testing, instance starts on the same
+    # network will have multi-second start delays due to lock contention .
+    with db.get_lock('network', None, uuid,
+                     ttl=10, timeout=120, op='Object load from DB'):
+        dbnet = db.get_network(uuid)
+        if not dbnet:
+            return None
+        if dbnet['state'] in ('deleted', 'deleting'):
+            LOG.withNetwork(uuid).info('Network is deleted, returning None.')
+            return None
+        return Network(dbnet)
 
 
 class Network(object):
@@ -33,7 +41,7 @@ class Network(object):
     # on Linux is 15 user visible characters.
     def __init__(self, db_entry):
         self.db_entry = db_entry
-        self.physical_nic = config.parsed.get('NODE_EGRESS_NIC')
+        self.physical_nic = config.get('NODE_EGRESS_NIC')
 
         with db.get_lock('ipmanager', None, self.db_entry['uuid'], ttl=120,
                          op='Network object initialization'):
@@ -50,7 +58,8 @@ class Network(object):
             db.persist_ipmanager(self.db_entry['uuid'], ipm.save())
 
     def __str__(self):
-        return 'network(%s, vxid %s)' % (self.db_entry['uuid'], self.db_entry['vxid'])
+        return 'network(%s, vxid %s)' % (self.db_entry['uuid'],
+                                         self.db_entry['vxid'])
 
     def unique_label(self):
         return ('network', self.db_entry['uuid'])
@@ -64,7 +73,7 @@ class Network(object):
             'vx_veth_inner': 'veth-%s-i' % self.db_entry['vxid'],
 
             'physical_interface': self.physical_nic,
-            'physical_bridge': 'phy-br-%s' % config.parsed.get('NODE_EGRESS_NIC'),
+            'physical_bridge': 'phy-br-%s' % config.get('NODE_EGRESS_NIC'),
             'physical_veth_outer': 'phy-%s-o' % self.db_entry['vxid'],
             'physical_veth_inner': 'phy-%s-i' % self.db_entry['vxid'],
 
@@ -105,29 +114,49 @@ class Network(object):
 
         return True
 
+    def is_dead(self):
+        """Check if the network is deleted or being deleted, or in error.
+
+        First, update the object model to the ensure latest configuration. Some
+        callers will wait on a lock before calling this function. In this case
+        we definitely need to update the in-memory object model.
+        """
+        # TODO(andy): To be improved when object model mirrors Image class
+        self.db_entry = db.get_network(self.db_entry['uuid'])
+
+        return self.db_entry['state'] in ('deleted', 'deleting', 'error')
+
     def create(self):
         subst = self.subst_dict()
 
         with db.get_object_lock(self, ttl=120, op='Network create'):
+            # Ensure network was not deleted whilst waiting for the lock.
+            if self.is_dead():
+                raise DeadNetwork('network=%s' % self)
+
             if not util.check_for_interface(subst['vx_interface']):
                 with util.RecordedOperation('create vxlan interface', self):
                     util.create_interface(
                         subst['vx_interface'], 'vxlan',
-                        'id %(vx_id)s dev %(physical_interface)s dstport 0' % subst)
+                        'id %(vx_id)s dev %(physical_interface)s dstport 0'
+                        % subst)
                     util.execute(None,
-                                 'sysctl -w net.ipv4.conf.%(vx_interface)s.arp_notify=1' % subst)
+                                 'sysctl -w net.ipv4.conf.'
+                                 '%(vx_interface)s.arp_notify=1' % subst)
 
             if not util.check_for_interface(subst['vx_bridge']):
                 with util.RecordedOperation('create vxlan bridge', self):
                     util.create_interface(subst['vx_bridge'], 'bridge', '')
                     util.execute(None,
-                                 'ip link set %(vx_interface)s master %(vx_bridge)s' % subst)
+                                 'ip link set %(vx_interface)s '
+                                 'master %(vx_bridge)s' % subst)
                     util.execute(None,
                                  'ip link set %(vx_interface)s up' % subst)
                     util.execute(None,
                                  'ip link set %(vx_bridge)s up' % subst)
                     util.execute(None,
-                                 'sysctl -w net.ipv4.conf.%(vx_bridge)s.arp_notify=1' % subst)
+                                 'sysctl -w net.ipv4.conf.'
+                                 '%(vx_bridge)s.arp_notify=1' % subst)
                     util.execute(None,
                                  'brctl setfd %(vx_bridge)s 0' % subst)
                     util.execute(None,
@@ -146,16 +175,21 @@ class Network(object):
                     util.create_interface(
                         subst['vx_veth_outer'], 'veth',
                         'peer name %(vx_veth_inner)s' % subst)
-                    util.execute(None,
-                                 'ip link set %(vx_veth_inner)s netns %(netns)s' % subst)
-                    util.execute(None,
-                                 'brctl addif %(vx_bridge)s %(vx_veth_outer)s' % subst)
+                    util.execute(
+                        None,
+                        'ip link set %(vx_veth_inner)s netns %(netns)s' % subst)
+                    util.execute(
+                        None,
+                        'brctl addif %(vx_bridge)s %(vx_veth_outer)s' % subst)
                     util.execute(None,
                                  'ip link set %(vx_veth_outer)s up' % subst)
-                    util.execute(None,
-                                 '%(in_netns)s ip link set %(vx_veth_inner)s up' % subst)
-                    util.execute(None,
-                                 '%(in_netns)s ip addr add %(router)s/%(netmask)s dev %(vx_veth_inner)s' % subst)
+                    util.execute(
+                        None,
+                        '%(in_netns)s ip link set %(vx_veth_inner)s up' % subst)
+                    util.execute(
+                        None,
+                        '%(in_netns)s ip addr add %(router)s/%(netmask)s '
+                        'dev %(vx_veth_inner)s' % subst)
 
             if not util.check_for_interface(subst['physical_veth_outer']):
                 with util.RecordedOperation('create physical veth', self):
@@ -163,11 +197,14 @@ class Network(object):
                         subst['physical_veth_outer'], 'veth',
                         'peer name %(physical_veth_inner)s' % subst)
                     util.execute(None,
-                                 'brctl addif %(physical_bridge)s %(physical_veth_outer)s' % subst)
+                                 'brctl addif %(physical_bridge)s '
+                                 '%(physical_veth_outer)s' % subst)
                     util.execute(None,
-                                 'ip link set %(physical_veth_outer)s up' % subst)
+                                 'ip link set %(physical_veth_outer)s up'
+                                 % subst)
                     util.execute(None,
-                                 'ip link set %(physical_veth_inner)s netns %(netns)s' % subst)
+                                 'ip link set %(physical_veth_inner)s '
+                                 'netns %(netns)s' % subst)
 
             self.deploy_nat()
             self.update_dhcp()
@@ -182,7 +219,8 @@ class Network(object):
 
         subst = self.subst_dict()
         if not self.db_entry['floating_gateway']:
-            with db.get_lock('ipmanager', None, 'floating', ttl=120, op='Network deploy NAT'):
+            with db.get_lock('ipmanager', None,
+                             'floating', ttl=120, op='Network deploy NAT'):
                 ipm = db.get_ipmanager('floating')
                 self.db_entry['floating_gateway'] = ipm.get_random_free_address()
                 db.persist_ipmanager('floating', ipm.save())
@@ -195,41 +233,56 @@ class Network(object):
         subst['floating_netmask'] = ipm.netmask
 
         with db.get_object_lock(self, ttl=120, op='Network deploy NAT'):
+            # Ensure network was not deleted whilst waiting for the lock.
+            if self.is_dead():
+                raise DeadNetwork('network=%s' % self)
+
             with util.RecordedOperation('enable virtual routing', self):
-                if not subst['floating_gateway'] in list(util.get_interface_addresses(
-                        subst['netns'], subst['physical_veth_inner'])):
+                addresses = util.get_interface_addresses(
+                    subst['netns'],
+                    subst['physical_veth_inner'])
+                if not subst['floating_gateway'] in list(addresses):
                     util.execute(None,
-                                 '%(in_netns)s ip addr add %(floating_gateway)s/%(floating_netmask)s '
+                                 '%(in_netns)s ip addr add '
+                                 '%(floating_gateway)s/%(floating_netmask)s '
                                  'dev %(physical_veth_inner)s' % subst)
                     util.execute(None,
-                                 '%(in_netns)s ip link set %(physical_veth_inner)s up' % subst)
+                                 '%(in_netns)s ip link set '
+                                 '%(physical_veth_inner)s up' % subst)
 
                 default_routes = util.get_default_routes(subst['netns'])
                 if default_routes != [subst['floating_router']]:
                     if default_routes:
                         for default_route in default_routes:
                             util.execute(None,
-                                         '%s route del default gw %s' % (subst['in_netns'], default_route))
+                                         '%s route del default gw %s'
+                                         % (subst['in_netns'], default_route))
 
                     util.execute(None,
-                                 '%(in_netns)s route add default gw %(floating_router)s' % subst)
+                                 '%(in_netns)s route add default '
+                                 'gw %(floating_router)s' % subst)
 
             if not util.nat_rules_for_ipblock(self.network_address):
                 with util.RecordedOperation('enable nat', self):
                     util.execute(None,
                                  'echo 1 > /proc/sys/net/ipv4/ip_forward')
                     util.execute(None,
-                                 '%(in_netns)s iptables -A FORWARD -o %(physical_veth_inner)s '
+                                 '%(in_netns)s iptables -A FORWARD '
+                                 '-o %(physical_veth_inner)s '
                                  '-i %(vx_veth_inner)s -j ACCEPT' % subst)
                     util.execute(None,
-                                 '%(in_netns)s iptables -A FORWARD -i %(physical_veth_inner)s '
+                                 '%(in_netns)s iptables -A FORWARD '
+                                 '-i %(physical_veth_inner)s '
                                  '-o %(vx_veth_inner)s -j ACCEPT' % subst)
                     util.execute(None,
-                                 '%(in_netns)s iptables -t nat -A POSTROUTING -s %(ipblock)s/%(netmask)s '
-                                 '-o %(physical_veth_inner)s -j MASQUERADE' % subst)
+                                 '%(in_netns)s iptables -t nat -A POSTROUTING '
+                                 '-s %(ipblock)s/%(netmask)s '
+                                 '-o %(physical_veth_inner)s '
+                                 '-j MASQUERADE' % subst)
 
     def delete(self):
         subst = self.subst_dict()
+        LOG.withFields(subst).debug('net.delete()')
 
         # Cleanup local node
         with db.get_object_lock(self, ttl=120, op='Network delete'):
@@ -252,7 +305,8 @@ class Network(object):
                 if util.check_for_interface(subst['physical_veth_outer']):
                     with util.RecordedOperation('delete physical veth', self):
                         util.execute(
-                            None, 'ip link delete %(physical_veth_outer)s' % subst)
+                            None,
+                            'ip link delete %(physical_veth_outer)s' % subst)
 
                 if os.path.exists('/var/run/netns/%(netns)s' % subst):
                     with util.RecordedOperation('delete netns', self):
@@ -318,6 +372,10 @@ class Network(object):
 
     def ensure_mesh(self):
         with db.get_object_lock(self, ttl=120, op='Network ensure mesh'):
+            # Ensure network was not deleted whilst waiting for the lock.
+            if self.is_dead():
+                raise DeadNetwork('network=%s' % self)
+
             removed = []
             added = []
 
@@ -340,7 +398,7 @@ class Network(object):
             # NOTE(mikal): why not use DNS here? Well, DNS might be outside
             # the control of the deployer if we're running in a public cloud
             # as an overlay cloud...
-            node_ips = [config.parsed.get('NETWORK_NODE_IP')]
+            node_ips = [config.NETWORK_NODE_IP]
             for fqdn in node_fqdns:
                 ip = db.get_node(fqdn)['ip']
                 if ip not in node_ips:
@@ -362,27 +420,29 @@ class Network(object):
                 added.append(node)
 
             if removed:
-                db.add_event('network', self.db_entry['uuid'], 'remove mesh elements',
-                             None, None, ' '.join(removed))
+                db.add_event(
+                    'network', self.db_entry['uuid'], 'remove mesh elements',
+                    None, None, ' '.join(removed))
             if added:
-                db.add_event('network', self.db_entry['uuid'], 'add mesh elements',
-                             None, None, ' '.join(added))
+                db.add_event(
+                    'network', self.db_entry['uuid'], 'add mesh elements',
+                    None, None, ' '.join(added))
 
     def _add_mesh_element(self, node):
         LOG.withObj(self).info('Adding new mesh element %s' % node)
         subst = self.subst_dict()
         subst['node'] = node
         util.execute(None,
-                     'bridge fdb append to 00:00:00:00:00:00 dst %(node)s dev %(vx_interface)s'
-                     % subst)
+                     'bridge fdb append to 00:00:00:00:00:00 '
+                     'dst %(node)s dev %(vx_interface)s' % subst)
 
     def _remove_mesh_element(self, node):
         LOG.withObj(self).info('Removing excess mesh element %s' % node)
         subst = self.subst_dict()
         subst['node'] = node
         util.execute(None,
-                     'bridge fdb del to 00:00:00:00:00:00 dst %(node)s dev %(vx_interface)s'
-                     % subst)
+                     'bridge fdb del to 00:00:00:00:00:00 dst %(node)s '
+                     'dev %(vx_interface)s' % subst)
 
     def add_floating_ip(self, floating_address, inner_address):
         LOG.withObj(self).info('Adding floating ip %s -> %s'
@@ -396,7 +456,8 @@ class Network(object):
                      'dev %(physical_veth_outer)s' % subst)
         util.execute(None,
                      '%(in_netns)s iptables -t nat -A PREROUTING '
-                     '-d %(floating_address)s -j DNAT --to-destination %(inner_address)s' % subst)
+                     '-d %(floating_address)s '
+                     '-j DNAT --to-destination %(inner_address)s' % subst)
 
     def remove_floating_ip(self, floating_address, inner_address):
         LOG.withObj(self).info('Removing floating ip %s -> %s'
@@ -410,4 +471,5 @@ class Network(object):
                      'dev %(physical_veth_outer)s' % subst)
         util.execute(None,
                      '%(in_netns)s iptables -t nat -D PREROUTING '
-                     '-d %(floating_address)s -j DNAT --to-destination %(inner_address)s' % subst)
+                     '-d %(floating_address)s '
+                     '-j DNAT --to-destination %(inner_address)s' % subst)
